// All database related API calls

import { useAuthorizationCache, useSessionCache } from '../store';

export const useSchemaAPI = () => {
  const cache = useSessionCache();
  const { accessToken } = useAuthorizationCache();
  const domain = import.meta.env.VITE_BACKEND_URL;
  const schema = import.meta.env.VITE_BACKEND_SCHEMA;

  async function RequestSchema(databaseName?: string) {
    if (!databaseName) databaseName = cache.currentDatabase;
    if (!databaseName) {
      throw Error('Must call with a database name');
    }

    const request = {
      databaseName,
      cached: true,
    };

    const response = await fetch(`${domain}${schema}/`, {
      method: 'POST',
      credentials: 'same-origin',
      headers: new Headers({
        Authorization: 'Bearer ' + accessToken,
      }),
      body: JSON.stringify(request),
    });

    if (!response?.ok) {
      const ret = await response.text();
      console.error(response, ret);
    }
    // const ret = await response.json();
    console.log(response);
  }

  return { RequestSchema };
};
