import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { MultiGraph } from 'graphology';
import { Attributes, SerializedGraph } from 'graphology-types';
import { QueryMultiGraph, QueryMultiGraphology } from '../../querybuilder';

// Define the initial state using that type
export const initialState = {
  graphologySerialized: new QueryMultiGraphology().export(),
  // schemaLayout: 'Graphology_noverlap',
};

export const querybuilderSlice = createSlice({
  name: 'querybuilder',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    setQuerybuilderNodes: (state, action: PayloadAction<SerializedGraph<Attributes, Attributes, Attributes>>) => {
      // console.log('setQuerybuilderNodes', action.payload);
      state.graphologySerialized = QueryMultiGraphology.from(action.payload).export() as QueryMultiGraph;
    },
    // updateQBAttributeOperator: (state, action: PayloadAction<{ id: string; operator: string }>) => {
    //   const graph = QueryMultiGraphology.from(state.graphologySerialized);
    //   graph.setNodeAttribute(action.payload.id, 'operator', action.payload.operator);
    //   state.graphologySerialized = graph.export();
    // },
    // updateQBAttributeValue: (state, action: PayloadAction<{ id: string; value: string }>) => {
    //   const graph = QueryMultiGraphology.from(state.graphologySerialized);
    //   graph.setNodeAttribute(action.payload.id, 'value', action.payload.value);
    //   state.graphologySerialized = graph.export();
    // },
    clearQB: (state) => {
      state.graphologySerialized = new QueryMultiGraphology().export();
    },

    // addQuerybuilderNode: (
    //   state,
    //   action: PayloadAction<{ id: string; attributes: Attributes }>
    // ) => {
    //   const graph = MultiGraph.from(state.graphologySerialized);
    //   graph.addNode(action.payload.id, action.payload.attributes);
    //   state.graphologySerialized = graph.export();
    // },
    // setGraphLayout: (state, action: PayloadAction<AllLayoutAlgorithms>) => {
    //   state.schemaLayout = action.payload;
    // },
  },
});

export const { setQuerybuilderNodes, clearQB } = querybuilderSlice.actions;

/** Select the querybuilder nodes in serialized fromat */
export const selectQuerybuilderGraphology = (state: RootState): QueryMultiGraphology => {
  // This is really weird but for some reason all the attributes appeared as read-only otherwise

  let ret = new QueryMultiGraphology();
  ret.import(MultiGraph.from(state.querybuilder.graphologySerialized).export());
  return ret;
};

/** Select the querybuilder nodes and convert it to a graphology object */
export const selectQuerybuilderGraph = (state: RootState): QueryMultiGraph => {
  // This is really weird but for some reason all the attributes appeared as read-only otherwise
  return state.querybuilder.graphologySerialized as QueryMultiGraph;
};

/** Select the querybuilder nodes and convert it to a graphology object */
export const selectQuerybuilderHash = (state: RootState): any => {
  return JSON.stringify({
    nodes: state.querybuilder.graphologySerialized.nodes.map((n) => ({
      key: n.key,
    })),
    edges: state.querybuilder.graphologySerialized.edges.map((n) => ({
      key: n.key,
      source: n.source,
      target: n.target,
    })),
  });
};

// /**
//  * selects the SchemaLayout enum
//  * @param {GraphLayout} state
//  * @returns {GraphLayout} enum of type GraphLayout
//  */
// export const selectSchemaLayout = (state: RootState) =>
//   state.schema.schemaLayout;

export default querybuilderSlice.reducer;
