import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { BackendQueryFormat } from '../../querybuilder';

/** Message format of the error message from the backend */
export type ErrorMessage = {
  errorStatus: any;
  queryID: any;
};

/** Cache type */
export type SessionCacheI = {
  currentDatabase?: string;
  databases: string[];
  error?: ErrorMessage;

  // queryListOpen: boolean;
  // queryStatusList: {
  //   queries: Record<string, QueryStatusListItem>;
  //   queryIDsOrder: string[];
  // };
  // functionsMenuOpen: boolean;
  // currentDatabaseKey: string;
  // currentColours: { key: string; index: number };
  // elementsperDatabaseObject: Record<string, (AnyNode | Edge<any>)[]>;
  // autoSendQueries: boolean;
  // panelWidthHeights: {
  //   windowinnerHeight: number;
  //   windowinnerWidth: number;
  //   navBarHeight: number;
  //   schemaDrawerHeight: number;
  //   queryDrawerHeight: number;
  //   schemaDrawerWidth: number;
  //   queryDrawerWidth: number;
  // };
};

// Define the initial state using that type
export const initialState: SessionCacheI = {
  currentDatabase: undefined,
  databases: [],
  error: undefined,
};

export const sessionSlice = createSlice({
  name: 'session',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    updateCurrentDatabase(state, action: PayloadAction<string>) {
      state.currentDatabase = action.payload;
    },
    updateDatabaseList(state, action: PayloadAction<string[]>) {
      console.log('Updating database list', action);
      state.databases = action.payload;
      if (state.databases.length > 0) {
        if (!state.currentDatabase || !state.databases.includes(state.currentDatabase)) state.currentDatabase = state.databases[0];
        else state.currentDatabase = undefined;
      }
    },
    displayError(state, action: PayloadAction<ErrorMessage>): void {
      // use a switch, in case certain errors will have side effects in the future
      switch (action.payload.errorStatus) {
        case 'Bad request':
        case 'Bad credentials':
        case 'Translation error':
        case 'Database error':
        case 'ML bad request':
          state.error = action.payload;
          break;
      }
    },
    closeError(state): void {
      state.error = undefined;
    },
  },
});

export const { updateCurrentDatabase, updateDatabaseList, displayError, closeError } = sessionSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const sessionCacheState = (state: RootState) => state.sessionCache;

export default sessionSlice.reducer;
