import { NodeAttribute, QueryGraphNodes } from './graphology';
import { SchemaReactflowNode } from './reactflow';

export * from './BackendQueryFormat';
export * from './graphology';
export * from './logic';
export * from './reactflow';

type ExtraProps = { extra?: string; separator?: string };
export function getHandleId(
  nodeId: string,
  nodeName: string,
  nodeType: string,
  attributeName: string,
  attributeType: string,
  { extra, separator }: ExtraProps = {}
): string {
  if (!extra) extra = '';
  if (!separator) separator = '__';
  return [nodeId, nodeType, nodeName, attributeName, attributeType, extra].join(separator);
}
export function getHandleIdFromGraphology(node: QueryGraphNodes, attribute: NodeAttribute, options: ExtraProps = {}): string {
  return getHandleId(node.id || '', node.name, node.type, attribute.name, attribute.type, options);
}
export function getHandleIdFromReactflow(node: SchemaReactflowNode, attribute: NodeAttribute, options: ExtraProps = {}): string {
  return getHandleId(node.id, node.data.name, node.type, attribute.name, attribute.type, options);
}
export function fromHandleId(
  handleId: string,
  separator: string = '__'
): { nodeId: string; nodeName: string; nodeType: string; attributeName: string; attributeType: string } {
  const [nodeId, nodeType, nodeName, attributeName, attributeType] = handleId.split(separator);
  return { nodeId, nodeType, nodeName, attributeName, attributeType };
}
