// import { handles } from '@graphpolaris/shared/lib/querybuilder/usecases';
import { useTheme } from '@mui/material';
import React, { MouseEventHandler, useEffect } from 'react';
import { ReactFlow, Handle, Position, getConnectedEdges } from 'reactflow';
import styles from './entitypill.module.scss';
import { SchemaReactflowEntityNode, Handles, getHandleIdFromReactflow } from '../../../model';
import { SchemaAttribute } from '@graphpolaris/shared/lib/schema';
import { styleHandleMap } from '../../utils';
import { useQuerybuilderGraph } from '@graphpolaris/shared/lib/data-access';

/**
 * Component to render an entity flow element
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const EntityFlowElement = React.memo((node: SchemaReactflowEntityNode) => {
  const theme = useTheme();
  const data = node.data;
  const forceOpen: boolean = true;

  const graph = useQuerybuilderGraph();
  const myEdges = graph.edges.filter(
    (edge) => (edge.source === node.id || edge.target === node.id) && !edge?.attributes?.sourceHandle.includes(Handles.EntityRight) // no need to show if only the relation handle is connected
  );

  const [hovered, setHovered] = React.useState(false);
  const [handleBeingDragged, setHandleBeingDragged] = React.useState(-1);

  // TODO: Change flow element width when text overflows
  const animation = styles.entityFade; // TODO: Check if correct

  const onMouseEnter = (event: React.MouseEvent) => {
    setHovered(true);
  };

  const onMouseLeave = (event: React.MouseEvent) => {
    setHovered(false);
  };

  const onHandleMouseDown = (attribute: SchemaAttribute, i: number, event: React.MouseEvent) => {
    setHandleBeingDragged(i);
    window.addEventListener('mouseup', onHandleMouseUp, true);
  };

  const onHandleMouseUp = () => {
    setHandleBeingDragged(-1);
    window.removeEventListener('mouseup', onHandleMouseUp, true);
  };

  const onConnect = (params: any) => {
    console.log('EntityPill onConnect', params);
  };

  const showingDropdown = hovered || handleBeingDragged !== -1 || myEdges.length > 0;

  return (
    <div
      className={`${styles.entity} ${animation} query_builder-entity`}
      onMouseEnter={onMouseEnter}
      onMouseLeave={onMouseLeave}
      style={
        {
          // borderColor: theme.palette.custom.elements.entityBase[0],
        }
      }
    >
      <Handle
        // id={getHandleId(data.name, data.type, Handles.ToRelation, '')}
        id={data.leftRelationHandleId}
        type="target"
        position={Position.Left}
        className={styles.handle_to_relation}
      />
      <Handle
        // id={getHandleId(data.name, data.type, Handles.ToRelation, '')}
        id={data.rightRelationHandleId}
        type="source"
        position={Position.Right}
        className={styles.handle_to_relation}
      />
      {/* <Handle
        id={Handles.ToAttribute}
        type="target"
        position={Position.Bottom}
        className={
          styles.ToAttributeHandle +
          ' ' +
          (false ? styles.handleConnectedFill : '')
        }
      />
      <Handle
        id={Handles.ReceiveFunction}
        type="target"
        position={Position.Bottom}
        className={
          styles.ReceiveFunctionHandle +
          ' ' +
          (false ? styles.handleConnectedFill : '')
        }
      /> */}

      <div
        className={styles.title}
        style={{
          backgroundColor: theme.palette.custom.elements.entityBase[0],
        }}
      >
        {data.name}
      </div>
      {/* <div className={styles.entityWrapper}>
        <span className={styles.entitySpan}>{data.name}</span>
      </div> */}
      {data?.attributes && (
        <div className={styles.content + ' ' + (showingDropdown || forceOpen ? styles.content_display : '')}>
          {data.attributes
            .filter(
              (attribute, i) =>
                forceOpen ||
                hovered ||
                handleBeingDragged === i ||
                myEdges.some((edge) => edge?.attributes?.sourceHandle === getHandleIdFromReactflow(node, attribute))
            )
            .map((attribute, i) => (
              <div
                key={attribute.name + i}
                onMouseDown={(event: React.MouseEvent) => {
                  onHandleMouseDown(attribute, i, event);
                }}
              >
                {attribute.name}
                <Handle
                  id={getHandleIdFromReactflow(node, attribute)}
                  type="source"
                  position={Position.Right}
                  className={(styleHandleMap?.[attribute.type] || '') + ' ' + styles.io + ' ' + styles.io_right}
                ></Handle>
              </div>
            ))}
        </div>
      )}
    </div>
  );
});

EntityFlowElement.displayName = 'EntityFlowElement';

export default EntityFlowElement;
