/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React from 'react';
import { Handle, HandleType, NodeProps, Position } from 'reactflow';
import styles from './logicpill.module.scss';
import { AllLogicMap, Handles, SchemaReactflowLogicNode, fromHandleId, getHandleId, getHandleIdFromReactflow } from '../../../model';
import { Input } from '@mui/material';
import { styleHandleMap } from '../../utils';
import { useQuerybuilderGraph } from '@graphpolaris/shared/lib/data-access';
import { InputNode } from '../../../model/logic/general';

/**
 * Component to render an entity flow element
 * @param param0 Data of the flow element.
 */
export default function LogicPill(node: SchemaReactflowLogicNode) {
  const data = node.data;
  const logic = data.logic;
  const left = logic.inputs.filter((input) => input.position === Position.Left);
  const right = logic.inputs.filter((input) => input.position === Position.Right);

  const graph = useQuerybuilderGraph();
  const leftEdges = graph.edges.filter((edge) => edge.target === node.id);
  const rightEdges = graph.edges.filter((edge) => edge.source === node.id);

  function createHandles(side: InputNode[], positionSide: Position, handleType: HandleType) {
    let numOfInputs = 0;
    let ret = side.map((input, i) => {
      let inputTextBox = null;
      if (
        positionSide === Position.Left &&
        !leftEdges.some(
          (edge) =>
            edge?.attributes?.targetHandle ===
            getHandleId(data?.id || 'awkudgi', data.name, node.type, input.name, input.type.join(''), { extra: positionSide }) // TODO
        )
      ) {
        inputTextBox = (
          <Input
            className={styles.logicInput}
            // value={0}
            style={{ top: -5, transform: `translateY(-${i * 20}%)` }}
            onChange={(e) => {}}
          />
        );
        numOfInputs++;
      }
      return (
        <Handle
          type={handleType}
          position={positionSide}
          id={getHandleId(data?.id || 'awkudgi', data.name, node.type, input.name, input.type.join(''), { extra: positionSide })} // TODO
          key={input.name + input.type}
          // style={{ top: `${((i + 0.8) / (side.length + 0.6)) * 120}%` }}
          style={{ top: `${((i + 0.8) / (side.length + 0.6)) * 100}%` }}
          className={input.type.map((type) => styleHandleMap?.[type] || '').join(' ')}
        >
          {inputTextBox}
        </Handle>
      );
    });
    return { handles: ret, number: numOfInputs };
  }
  const { handles: leftHandles, number: leftInputsNumber } = createHandles(left, Position.Left, 'target');
  const { handles: rightHandles, number: rightInputsNumber } = createHandles(right, Position.Right, 'source');

  return (
    <div className={styles.logic}>
      <span className={styles.logicSpan} style={{ height: `${Math.max(leftInputsNumber * 2.6, rightInputsNumber * 2, 1)}rem` }}>
        {
          <span>
            {leftEdges.map((e) => fromHandleId(e?.attributes?.sourceHandle)?.attributeName).join(', ')}.{right?.[0].name}
          </span>
        }
        {leftHandles}
        {rightHandles}
      </span>
    </div>
  );
}
