import { EntityNodeAttributes, QueryGraphNodes, RelationNodeAttributes } from '../model/graphology/model';
import { QueryMultiGraph } from '../model/graphology/utils';
import { BackendQueryFormat, NodeStruct, QueryStruct, RelationStruct } from '../model/BackendQueryFormat';
import { Handles, QueryElementTypes, fromHandleId } from '../model';
import { get } from 'http';
import { SerializedEdge } from 'graphology-types';
import { G } from 'vitest/dist/types-fafda418';

// export type QueryI {

// }

/**
 * Converts the ReactFlow query to a json data structure to send the query to the backend.
 * @returns {BackendQueryFormat} A JSON object in the `JSONFormat`.
 */
export function Query2BackendQuery(databaseName: string, graph: QueryMultiGraph): BackendQueryFormat {
  let query: BackendQueryFormat = {
    databaseName: databaseName,
    query: [],
    limit: 5000, // TODO
    return: ['*'], // TODO
  };
  let graphLogicChunks: QueryGraphNodes[][] = [[]];

  const entities: EntityNodeAttributes[] = graph.nodes
    .map((n) => n.attributes)
    .filter((n) => n?.type === QueryElementTypes.Entity) as EntityNodeAttributes[];
  const entitiesEmptyLeftHandle = entities.filter((n) => !graph.edges.some((e) => e.target === n?.id));
  // const entitiesEmptyRightHandle = entities.filter((n) => !n?.rightRelationHandleId);

  const relations: RelationNodeAttributes[] = graph.nodes
    .map((n) => n.attributes)
    .filter((n) => n?.type === QueryElementTypes.Relation) as RelationNodeAttributes[];
  // const relationsEmptyLeftHandle = relations.filter((n) => !n?.leftEntityHandleId);
  // const relationsEmptyRightHandle = relations.filter((n) => !n?.rightEntityHandleId);

  const traversePaths = (node: QueryGraphNodes, paths: QueryGraphNodes[][], currentIdx: number): number => {
    if (!paths?.[currentIdx]) paths.push([]);
    paths[currentIdx].push(node);

    // const rightHandle =
    //   node.type === QueryElementTypes.Entity
    //     ? (node as EntityNodeAttributes)?.rightRelationHandleId
    //     : (node as RelationNodeAttributes)?.rightEntityHandleId;

    let connections = graph.edges.filter((e) => e.source === node.id);
    if (connections.length === 0) return 0;

    const nodesToRight = connections
      .map((c, i) => {
        const rightNodeHandleData = fromHandleId(c.attributes?.targetHandle);
        const rightNode =
          rightNodeHandleData.nodeType === QueryElementTypes.Entity
            ? entities.find((r) => r.id === c.target)
            : relations.find((r) => r.id === c.target);
        return rightNode;
      })
      .filter((n) => n !== undefined) as QueryGraphNodes[];

    // if (!rightNode) return;
    // if (currentIdx + i === paths.length) paths.push(paths[currentIdx - 1 + i]);
    // inverseTraverse(rightNode, paths, currentIdx + i);
    // paths[currentIdx + i].push(rightNode);
    // nodesToRight.forEach((rightNode, i) => {
    //   if (currentIdx + i === paths.length) {
    //     paths.push(paths[currentIdx - 1 + i]);
    //   }
    // });

    let chunkOffset = 0;
    nodesToRight.forEach((rightNode, i) => {
      chunkOffset += traversePaths(rightNode, paths, currentIdx + i + chunkOffset);
    });

    return nodesToRight.length - 1; // offset
  };

  // find paths which ends at an entity with no right handle
  let chunkOffset = 0;
  entitiesEmptyLeftHandle.map((entity, i) => {
    chunkOffset += traversePaths(entity, graphLogicChunks, i + chunkOffset);
  });

  console.log('graphLogicChunks', graphLogicChunks);
  if (!graphLogicChunks || graphLogicChunks.length === 0 || graphLogicChunks?.[0].length === 0) return query;

  const processConnection = (chunk: QueryGraphNodes[], position: number): RelationStruct | NodeStruct => {
    const currNode = chunk[position];

    if (currNode.type === QueryElementTypes.Relation) {
      const ret: RelationStruct = {
        ID: currNode?.id,
        label: currNode?.name,
        // depth: QuerySearchDepthStruct;
        direction: 'TO',
        node: chunk.length === position + 1 ? undefined : (processConnection(chunk, position + 1) as NodeStruct),
      };
      return ret;
    } else if (currNode.type === QueryElementTypes.Entity) {
      const ret: NodeStruct = {
        ID: currNode?.id,
        label: currNode?.name,
        // logic: LogicStruct[];
        // subQuery?: QueryStruct;
        // export: ExportNodeStruct[];
        relation: chunk.length === position + 1 ? undefined : (processConnection(chunk, position + 1) as RelationStruct),
      };
      return ret;
    }

    throw Error('Malformed Chunks! ' + chunk + position);
  };

  query.query = graphLogicChunks.map((chunk, i) => {
    const ret: QueryStruct = {
      ID: 'path_' + i, //TODO: chunk[0].id ||
      node: processConnection(chunk.reverse(), 0),
    };
    return ret;
  });

  console.log(query);

  return query;
}

// function processConnectionFromRelation(
//   c: SerializedEdge,
//   entities: EntityNodeAttributes[],
//   relations: RelationNodeAttributes[]
// ): NodeStruct {
//   if (c.attributes?.targetHandle === null) throw Error('Malformed Graph! One or more edges of a relation node do not exist');
//   const targetHandleData = fromHandleId(c.attributes?.targetHandle);

//   if (targetHandleData.nodeType === QueryElementTypes.Entity) {
//     const targetEntity = entities.find((r) => r.id === c.target);
//   } else if (targetHandleData.nodeType === QueryElementTypes.Relation) {
//     const targetRelation = relations.find((r) => r.id === c.target);
//   } else if (targetHandleData.nodeType === QueryElementTypes.Logic) {
//   }
// }
