import { AllLayoutAlgorithms, LayoutFactory } from '@graphpolaris/shared/lib/graph-layout';
import { schemaGraphology2Reactflow, schemaExpandRelation } from '@graphpolaris/shared/lib/schema/schema-utils';
import { useSchemaGraphology, useSchemaLayout } from '@graphpolaris/shared/lib/data-access/store';
import { MultiGraph } from 'graphology';
// import { AllLayoutAlgorithms, LayoutFactory } from '@graphpolaris/graph-layout';
import { useEffect, useMemo, useState } from 'react';
import ReactFlow, { ControlButton, Controls, Node, Edge, ReactFlowProvider, useNodesState, useEdgesState } from 'reactflow';

import 'reactflow/dist/style.css';

import styles from './schema.module.scss';

import {
  EntityFlowElement,
  RelationPill,
  AttributePill,
  ConnectionDragLine,
  ConnectionLine,
} from '@graphpolaris/shared/lib/querybuilder/pills';

interface Props {
  content?: string;
}

const onLoad = (reactFlowInstance: any) => {
  setTimeout(() => reactFlowInstance.fitView(), 100);
};

const nodeTypes = {
  entity: EntityFlowElement,
  relation: RelationPill,
  attribute: AttributePill,
};
const edgeTypes = {
  connection: ConnectionLine,
};

export const Schema = (props: Props) => {
  const [nodes, setNodes, onNodeChanged] = useNodesState([] as Node[]);
  const [edges, setEdges, onEdgeChanged] = useEdgesState([] as Edge[]);
  // In case the schema is updated
  const dbschema = useSchemaGraphology();
  // const [dbschema, setSchema] = useState(useSchema());
  const [schemaLayout, setSchemaLayout] = useState(useSchemaLayout());

  // useEffect(() => {
  //   console.log('dbSchema', dbschema, dbschema.order);
  // }, [dbschema]);

  const expandedSchema = useMemo(() => schemaExpandRelation(dbschema), [dbschema]);

  useEffect(() => {
    if (dbschema == undefined || dbschema.order == 0) {
      return;
    }

    const layoutFactory = new LayoutFactory();
    console.log('schema Layout', schemaLayout, 'order', expandedSchema);
    const layout = layoutFactory.createLayout(schemaLayout as AllLayoutAlgorithms);
    layout?.layout(expandedSchema);

    const flowElements = schemaGraphology2Reactflow(expandedSchema);
    setNodes(flowElements.nodes);
    setEdges(flowElements.edges);
    console.log('update schema useEffect', dbschema, dbschema.order, flowElements);
  }, [dbschema, schemaLayout]);

  const graphStyles = { width: '100%', height: '500px' };

  // console.log(nodes, edges);

  return (
    <div
      style={{
        width: '100%',
        height: '100%',
      }}
    >
      {nodes.length === 0 && <p>DEBUG: No Elements</p>}
      <ReactFlowProvider>
        <ReactFlow
          className={styles.schemaPanel}
          onlyRenderVisibleElements={false}
          nodesDraggable={false}
          nodeTypes={nodeTypes}
          connectionLineComponent={ConnectionDragLine}
          onNodesChange={onNodeChanged}
          onEdgesChange={onEdgeChanged}
          nodes={nodes}
          edges={edges}
          style={graphStyles}
          onLoad={onLoad}
          attributionPosition="top-right"
        >
          <Controls showInteractive={false} showZoom={false} showFitView={true} className={styles.controls}>
            <ControlButton
              className={styles.exportButton}
              title={'Export graph schema'}
              onClick={(event) => {
                event.stopPropagation();
                // this.setState({
                //   ...this.state,
                //   exportMenuAnchor: event.currentTarget,
                // });
              }}
            >
              {/* <img src={exportIcon} width={21}></img> */}
            </ControlButton>
          </Controls>
        </ReactFlow>
      </ReactFlowProvider>
    </div>
  );
};

export default Schema;

// Fix layout of the schema
// create reactflow elements on xy coords
// connect reactflow elements together

// maybe ook gelijk instellingen knoppie fixen op alle panels
