/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { SchemaFromBackend } from '@graphpolaris/shared/lib/schema';
import { EntitiesFromSchema } from '../Types';
import { GraphQueryResult } from '@graphpolaris/shared/lib/data-access';

/** Use case for retrieving entity names and attribute names from a schema result. */
export default class CalcEntityAttrNamesFromResultUseCase {
  /**
   * Takes a schema result and calculates all the entity names and attribute names per datatype.
   * Used by semantic substrates for the plot titles and add plot selections.
   * @param {SchemaFromBackend} schemaResult A new schema result from the backend.
   * @param {EntitiesFromSchema} All entity names and attribute names per datatype. So we know what is in the Schema.
   * @returns {EntitiesFromSchema} All entity names and attribute names per datatype.
   * @deprecated //TODO remove
   */
  public static CalcEntityAttrNamesFromResult(
    nodeLinkResult: GraphQueryResult,
    entitiesFromSchema: EntitiesFromSchema
  ): EntitiesFromSchema {
    const listOfNodeTypes: string[] = [];

    nodeLinkResult.nodes.forEach((node) => {
      let entityName = node.label;
      if (!listOfNodeTypes.includes(entityName)) {
        listOfNodeTypes.push(entityName);
      }
    });

    let entitiesFromSchemaPruned: EntitiesFromSchema = {
      entityNames: [],
      attributesPerEntity: {},
    };
    Object.assign(entitiesFromSchemaPruned, entitiesFromSchema);

    entitiesFromSchemaPruned.entityNames = listOfNodeTypes;
    return entitiesFromSchemaPruned;
  }
}
